@@ -31,7 +31,7 @@
 #include <iostream>
 
 #ifdef MAEMO
-static  osso_context_t *osso_context = NULL;
+osso_context_t *osso_context = NULL;
 #endif
 
 
