@@ -22,10 +22,12 @@
 
 #include "otrplayer.h"
 #include "otrplayerfile.h"
+#include "otrplayer_thread.h"
 #include "id3tag.h"
 
 #include <QtGui>
 #include <Qt3Support>
+#include <QSize>
 
 #include "MyQ3ListViewItem.h"
 
@@ -51,6 +53,7 @@
 
 using std::list;
 
+#define NUM_THREADS 2
 
 #define PLAY_DELTA 1024 * 4
 #define MP3_DATA_CFG "otrplayer.cfg"
@@ -65,6 +68,13 @@ using std::list;
 #define RATE_FIELD   4
 #define DIR_FIELD    5
 
+#ifdef MAEMO
+#include <libosso.h>
+#if 0
+#include <osso-multimedia-interface.h>
+#endif
+extern osso_context_t *osso_context; 
+#endif
 
 
 // Constructor
@@ -166,7 +176,7 @@ otrplayer::about()
       std::cout << "otrplayer: about called" << std::endl;
    QMessageBox::about(this, tr("About Otrplayer"),
       tr("<img src=\"/usr/share/icons/hicolor/scalable/hildon/otrplayer.png\" align=\"middle\" />"
-      "<h1 align=\"center\"><b>Otrplayer 0.42</b></h1>"
+      "<h1 align=\"center\"><b>Otrplayer 0.50</b></h1>"
       "<h5 align=\"center\">OTR Audio Player</h5>"
       "<h5 align=\"center\">(C) 2009 Bruce Forsberg</h5>"
       "<h5 align=\"center\">http://otrplayer.garage.maemo.org</h5>"));
@@ -234,6 +244,7 @@ otrplayer::performKeyEvent(int func_event)
 {
    int  value;
    bool action = true;
+   bool ret_value;
 
 
    // No Selection
@@ -248,13 +259,17 @@ otrplayer::performKeyEvent(int func_event)
    // Fast Forward
    else if (func_event == 3)
    {
-      _otr_file->getAdvance(0, value);
+      ret_value = _otr_file->getAdvance(0, value);
+      if (ret_value == FALSE)
+         value = 30;
       seekSeconds(value);
    }
    // Fast Rewind
    else if (func_event == 4)
    {
-      _otr_file->getAdvance(1, value);
+      ret_value = _otr_file->getAdvance(1, value);
+      if (ret_value == FALSE)
+         value = -30;
       seekSeconds(value);
    }
    // Play
@@ -355,12 +370,8 @@ otrplayer::getFiles()
    QStringList  lst, tree;
    QStringList::Iterator  it, itt, it_off;
    MyQ3ListViewItem  *item = NULL;
-   aflibFile  *temp = NULL;
-   aflibConfig  input_config;
-   aflibStatus  status;
    int          value;
    QString      str;
-   bool         ret_value;
    QStringList  tmp_lst;
    QString      filepath;
    int          sec, total_sec;
@@ -372,6 +383,15 @@ otrplayer::getFiles()
    QString      dir_save;
    QString      file_save;
    bool         value_saved = FALSE;
+   int          j;
+
+
+   otrplayer_thread *thread[NUM_THREADS];
+   for (j = 0; j < NUM_THREADS; j++)
+   {
+      thread[j] = new otrplayer_thread();
+   }
+
 
    if (_debug)
       std::cout << "getFiles: Entering getFiles" << std::endl;
@@ -464,11 +484,20 @@ otrplayer::getFiles()
    if (_debug)
       std::cout << "getFiles: Total number of files are: " << file_list.size() << std::endl;
 
-   for (it = file_list.begin(), it_off = offset_list.begin();
-        it != file_list.end(); ++it, ++it_off)
+   bool continue_run = true;
+   it = file_list.begin();
+   it_off = offset_list.begin();
+   QString  str1, str2, str3;
+   while (continue_run)
    {
-      QString  str1, str2, str3;
-      QString  local_str = (*it).toLocal8Bit();
+      for (j = 0; it != file_list.end() && j < NUM_THREADS; ++it, ++it_off, j++)
+      {
+         thread[j]->setName(*it);
+         thread[j]->setOffset((*it_off).toLong());
+      }
+      if (j != NUM_THREADS)
+         continue_run = false;
+      int num_threads = j;
 
 #if 0
       if (_debug)
@@ -479,75 +508,36 @@ otrplayer::getFiles()
       }
 #endif
 
-      // IF an ogg extension then use AUTO type
-      if ((*it).endsWith(".mp3", Qt::CaseInsensitive) == true)
-         temp = aflibFile::open(AFLIB_MPEG_TYPE, local_str.toStdString(),
-               &input_config, &status);
-      else
-         temp = aflibFile::open(AFLIB_AUTO_TYPE, local_str.toStdString(),
-               &input_config, &status);
-
-      // Did we not find madplay
-      if (status == AFLIB_NOT_FOUND)
-         madplay_found = FALSE;
-
-      if ((status == AFLIB_ERROR_INITIALIZATION_FAILURE) ||
-          (status == AFLIB_ERROR_OPEN))
+      for (int jj = 0; jj < num_threads; jj++)
       {
-         QString str = "Can\'t recognize file ";
-         str.append((*it).toLatin1());
-         infoEdit->insertLine(str, -1);
+         thread[jj]->start();
       }
-      else
+      for (int jj = 0; jj < num_threads; jj++)
       {
-         if (temp)
-         {
-            ret_value = temp->getItem("kbps", (void *)&value);
-            if (ret_value == FALSE)
-               value = 0;
-         }
-         else
-         {
-            if (_debug)
-               std::cout << "getFiles: ERROR creating aflibAudioFile" << std::endl;
-            value = 0;
-         }
+         thread[jj]->wait();
+      }
 
-         if (input_config.getSamplesPerSecond() != 0)
-         {
-            total_sec = input_config.getTotalSamples()/input_config.getSamplesPerSecond();
-            // round up to next second
-            total_sec++;
-         }
-         else 
-            total_sec = 0;
+      for (int jj = 0; jj < num_threads; jj++)
+      {
+         QString& name = thread[jj]->getName();
+         value = thread[jj]->getKBPS();
+
+         total_sec = thread[jj]->getTotalSec();
          secondsToQString(total_sec, str1);
 
-         if (input_config.getSamplesPerSecond() != 0)
-         {
-            long off = (*it_off).toLong();
-
-            // IF -1 we are at end of file
-            if (off == -1)
-               sec = total_sec;
-            else
-               sec = off / input_config.getSamplesPerSecond();
-         }
-         else
-            sec = 0;
+         sec = thread[jj]->getSeconds();
          secondsToQString(sec, str2);
 
          str.setNum(value);
-         str3.setNum(input_config.getSamplesPerSecond());
+         str3.setNum(thread[jj]->getSamplesPerSecond());
 
          QChar chr('/');
          int slash;
-         slash = (*it).lastIndexOf(chr);
+         slash = name.lastIndexOf(chr);
          item = new MyQ3ListViewItem( playList, item,
-            (*it).right((*it).length() - slash - 1),
-            str2, str1, str, str3, (*it).left(slash));
+            name.right(name.length() - slash - 1),
+            str2, str1, str, str3, name.left(slash));
       }
-      delete temp;
    }
 
    // IF madplay was not found and have not displayed warning yet
@@ -591,6 +581,11 @@ otrplayer::getFiles()
          }
       }
    }
+
+   for (j = 0; j < NUM_THREADS; j++)
+   {
+      delete thread[j];
+   }
 }
 
 
@@ -662,8 +657,35 @@ otrplayer::stop()
 void
 otrplayer::slider(int value)
 {
-   _current_position = _total_size  * _sample_rate_conversion_factor / 100 * value;
-   _source_current_position = _total_size  * _sample_rate_conversion_factor / 100 * value;
+   QString str;
+   int sec;
+
+   _slider_value = value;
+
+   sec = _total_size * _sample_rate_conversion_factor * value / 100 /_samples_per_second;
+   secondsToQString(sec, str);
+   _item->setText(LENGTH_FIELD, str);
+}
+
+void
+otrplayer::sliderUpdate()
+{
+   if (_debug)
+   {
+      std::cout << "Slider old _current_position " << std::dec << _current_position << std::endl;
+      std::cout << "Slider old _source_current_position " << _source_current_position << std::endl;
+   }
+   long long delta = (_total_size / 100 * _slider_value) - _source_current_position;
+   _source_current_position += delta;
+   _current_position = _source_current_position;
+   stop();
+   play();
+   if (_debug)
+   {
+      std::cout << "Slider delta" << delta << std::endl;
+      std::cout << "Slider new _current_position " << _current_position << std::endl;
+      std::cout << "Slider new _source_current_position " << _source_current_position << std::endl;
+   }
 }
 
 
@@ -737,7 +759,7 @@ void
 otrplayer::seekSeconds(
    int sec)
 {
-   _source_current_position += (sec * _samples_per_second / _sample_rate_conversion_factor);
+   _source_current_position += (long long)(sec * _samples_per_second / _sample_rate_conversion_factor);
    if (_source_current_position < 0)
       _source_current_position = 0;
    _current_position = _source_current_position;
@@ -833,7 +855,7 @@ otrplayer::run()
          //std::cout << "Position: " << _current_position << " Delta: " << delta << std::endl;
 
          _current_position += delta;
-         _source_current_position += (delta / _sample_rate_conversion_factor);
+         _source_current_position += (long long)(delta / _sample_rate_conversion_factor);
 
          // Update time, only every second
          if (_samples_per_second == 0)
@@ -846,7 +868,7 @@ otrplayer::run()
             if (_total_size == 0)
                timeSlider->setValue(0);
             else
-               timeSlider->setValue((int)( 100 * _source_current_position * _sample_rate_conversion_factor / _total_size));
+               timeSlider->setValue((int)( 100 * _source_current_position / _total_size));
             _item->setText(LENGTH_FIELD, str);
          }
          old_sec = sec;
@@ -995,8 +1017,10 @@ otrplayer::initButtons()
    menu_sel = _otr_file->getButton(otrplayerfile::MENU_KEY);
    menuBox->setCurrentItem(menu_sel);
 
+#if 0
    menu_sel = _otr_file->getButton(otrplayerfile::EMAIL_KEY);
    emailBox->setCurrentItem(menu_sel);
+#endif
 
    // Fast Forward advance button value
    if (_otr_file->getAdvance(0, value) == true)
@@ -1026,23 +1050,16 @@ otrplayer::initButtons()
 void
 otrplayer::volumeChange(int value)
 {
-   int vol;
-   int new_vol;
+#if 0
+   static osso_rpc_t vol;
 
-   int fd = open("/dev/mixer", O_RDWR);
-   ioctl( fd , MIXER_READ(0), &vol );
+	vol.type = DBUS_TYPE_DOUBLE;
+	vol.value.d = 5.0;
 
-   new_vol = vol & 0xff;
-   new_vol += value;
-   if (new_vol > 100)
-      new_vol = 100;
-   else if (new_vol < 0)
-      new_vol = 0;
-
-   vol = new_vol < 8 | new_vol;
-   
-   ioctl( fd , MIXER_WRITE(0), &vol );
-   ::close( fd );
+	osso_rpc_run(osso_context,
+      OSSO_MULTIMEDIA_SERVICE, OSSO_MULTIMEDIA_OBJECT_PATH, OSSO_MULTIMEDIA_SOUND_INTERFACE,
+      OSSO_MULTIMEDIA_SET_VOLUME_REQ, &vol, DBUS_TYPE_DOUBLE, NULL, DBUS_TYPE_INVALID);
+#endif
 }
 
 
@@ -1073,13 +1090,13 @@ otrplayer::menuCallback(int value)
    _otr_file->setButton(otrplayerfile::MENU_KEY, value);
 }
 
-
+#if 0
 void
 otrplayer::emailCallback(int value)
 {
    _otr_file->setButton(otrplayerfile::EMAIL_KEY, value);
 }
-
+#endif
 
 void
 otrplayer::initDirs()
